"use strict";

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");

var _Object$defineProperty = require("@babel/runtime-corejs3/core-js-stable/object/define-property");

_Object$defineProperty(exports, "__esModule", {
  value: true
});

exports.default = void 0;

var _embarkI18n = require("embark-i18n");

var _embarkUtils = require("embark-utils");

var _server = _interopRequireDefault(require("./server"));

const DEFAULT_PORT = 55555;
const DEFAULT_HOSTNAME = "localhost";

class Api {
  constructor(embark, options) {
    this.embark = embark;
    this.options = options;
    this.port = void 0;
    this.api = void 0;
    this.apiUrl = void 0;
    this.isServiceRegistered = false;
    this.embark.events.emit("status", (0, _embarkI18n.__)("Starting API & Cockpit UI"));
    (0, _embarkUtils.findNextPort)(DEFAULT_PORT).then(port => {
      this.port = port;
      this.apiUrl = `http://${DEFAULT_HOSTNAME}:${this.port}`;
      this.api = new _server.default(this.embark, this.port, (0, _embarkUtils.dockerHostSwap)(DEFAULT_HOSTNAME), options.plugins);
      this.listenToCommands();
      this.registerConsoleCommands();
      this.init();
    });
  }

  init() {
    this.embark.events.request("processes:register", "api", {
      launchFn: cb => {
        this.api.start().then(() => cb(null, (0, _embarkI18n.__)("Cockpit UI available at %s", this.apiUrl))).catch(error => cb(error, ""));
      },
      stopFn: cb => {
        this.api.stop().then(() => cb(null, (0, _embarkI18n.__)("Cockpit UI stopped"))).catch(error => cb(error, ""));
      }
    });
    this.embark.events.request("processes:launch", "api", (error, message) => {
      if (error) {
        this.embark.logger.error(error.message);
      } else {
        this.embark.logger.info(message);
      }

      this.setServiceCheck();
    });
    this.embark.events.on("check:wentOffline:api", () => {
      this.embark.logger.info((0, _embarkI18n.__)("Cockpit is offline, please close Cockpit."));
    });
    this.embark.events.on("check:backOnline:api", () => {
      this.embark.logger.info((0, _embarkI18n.__)("Cockpit is online, please open/refresh Cockpit."));
    });
  }

  setServiceCheck() {
    if (this.isServiceRegistered) {
      return;
    }

    this.isServiceRegistered = true;
    this.embark.events.request("services:register", "api", cb => {
      (0, _embarkUtils.checkIsAvailable)(this.apiUrl, isAvailable => {
        const devServer = (0, _embarkI18n.__)("Cockpit UI") + " (" + this.apiUrl + ")";
        const serverStatus = isAvailable ? "on" : "off";
        return cb({
          name: devServer,
          status: serverStatus
        });
      });
    });
  }

  listenToCommands() {
    this.embark.events.setCommandHandler("api:url", cb => cb(this.apiUrl));
    this.embark.events.setCommandHandler("api:start", cb => {
      this.embark.logger.warn((0, _embarkI18n.__)("The event 'api:start' has been deprecated and will be removed in future versions."));
      this.embark.events.request("processes:launch", "api", cb);
    });
    this.embark.events.setCommandHandler("api:stop", cb => {
      this.embark.logger.warn((0, _embarkI18n.__)("The event 'api:stop' has been deprecated and will be removed in future versions."));
      this.embark.events.request("processes:stop", "api", cb);
    });
    this.embark.events.setCommandHandler("logs:api:enable", cb => {
      this.api.enableLogging();
      cb();
    });
    this.embark.events.setCommandHandler("logs:api:disable", cb => {
      this.api.disableLogging();
      cb();
    });
  }

  registerConsoleCommands() {
    this.embark.registerConsoleCommand({
      description: (0, _embarkI18n.__)("Start or stop the API"),
      matches: ["api start"],
      process: (cmd, callback) => {
        const message = (0, _embarkI18n.__)("The command 'api:start' has been deprecated in favor of 'service api on' and will be removed in future versions.");
        this.embark.logger.warn(message); // logs to Embark's console

        this.embark.events.request("processes:launch", "api", (err, msg) => {
          callback(err || msg); // logs to Cockpit's console
        });
      },
      usage: "api start/stop"
    });
    this.embark.registerConsoleCommand({
      matches: ["api stop"],
      process: (cmd, callback) => {
        const message = (0, _embarkI18n.__)("The command 'api:stop' has been deprecated in favor of 'service api off' and will be removed in future versions.");
        this.embark.logger.warn(message); // logs to Embark's console

        this.embark.events.request("processes:stop", "api", (err, msg) => {
          callback(err || msg); // logs to Cockpit's console
        });
      }
    });
    this.embark.registerConsoleCommand({
      matches: ["log api on"],
      process: (cmd, callback) => {
        this.embark.events.request("logs:api:enable", callback);
      }
    });
    this.embark.registerConsoleCommand({
      matches: ["log api off"],
      process: (cmd, callback) => {
        this.embark.events.request("logs:api:disable", callback);
      }
    });
  }

}

exports.default = Api;
//# sourceMappingURL=index.js.map