"use strict";

var _interopRequireWildcard = require("@babel/runtime-corejs3/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");

var _Object$defineProperty = require("@babel/runtime-corejs3/core-js-stable/object/define-property");

_Object$defineProperty(exports, "__esModule", {
  value: true
});

exports.default = void 0;

var _bind = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/bind"));

var _stringify = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/json/stringify"));

var _trim = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/trim"));

var _map = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/map"));

var _filter = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/filter"));

var _forEach = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/for-each"));

var _promise = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/promise"));

var _set = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/set"));

var _bodyParser = _interopRequireDefault(require("body-parser"));

require("colors");

var _cors = _interopRequireDefault(require("cors"));

var _embarkI18n = require("embark-i18n");

var _embarkUtils = require("embark-utils");

var _express = _interopRequireDefault(require("express"));

var _expressWs = _interopRequireDefault(require("express-ws"));

var _findUp = _interopRequireDefault(require("find-up"));

var _helmet = _interopRequireDefault(require("helmet"));

var path = _interopRequireWildcard(require("path"));

var ws = _interopRequireWildcard(require("ws"));

class Server {
  constructor(embark, port, hostname, plugins) {
    this.embark = embark;
    this.port = port;
    this.hostname = hostname;
    this.plugins = plugins;
    this.isInsideMonorepo = void 0;
    this.monorepoRootPath = "";
    this.embarkUiBuildDir = (0, _embarkUtils.embarkPath)("node_modules/embark-ui/build");
    this.expressInstance = void 0;
    this.isLogging = false;
    this.server = void 0;
    this.openSockets = new _set.default();
    this.isInsideMonorepo = (0, _embarkUtils.isInsideMonorepoSync)();

    if (this.isInsideMonorepo) {
      this.monorepoRootPath = (0, _embarkUtils.monorepoRootPathSync)();
    } // in the monorepo and other deduped installs embark-ui may be in a higher-up node_modules


    const foundEmbarkUi = _findUp.default.sync("node_modules/embark-ui", {
      cwd: (0, _embarkUtils.embarkPath)(),
      type: "directory"
    });

    if (foundEmbarkUi) {
      this.embarkUiBuildDir = path.join(foundEmbarkUi, "build");
    }

    this.expressInstance = this.initApp();
  }

  enableLogging() {
    this.isLogging = true;
  }

  disableLogging() {
    this.isLogging = false;
  }

  start() {
    return new _promise.default((resolve, reject) => {
      if (this.server) {
        const message = (0, _embarkI18n.__)("API is already running");
        return reject(new Error(message));
      }

      this.server = this.expressInstance.app.listen(this.port, this.hostname, () => {
        resolve();
      }); // keep track of our open websockets so we can destroy them
      // if the api server is shutdown

      this.server.on("connection", socket => {
        this.openSockets.add(socket);
        socket.on("close", () => {
          this.openSockets.delete(socket);
        });
      });
    });
  }

  stop() {
    return new _promise.default((resolve, reject) => {
      var _context;

      if (!this.server) {
        const message = (0, _embarkI18n.__)("API is not running");
        return reject(new Error(message));
      } // close any open sockets


      (0, _forEach.default)(_context = this.openSockets).call(_context, socket => socket.destroy());
      this.server.close(() => {
        this.server = undefined;
        resolve();
      });
    });
  }

  makePage(body) {
    var _context2, _context3, _context4;

    return (0, _filter.default)(_context2 = (0, _map.default)(_context3 = (0, _trim.default)(_context4 = `
      <!doctype html>
      <html lang="en">
        <head>
          <meta charset="utf-8" />
          <title>Embark API Server</title>
          <style type="text/css">
            code {
              background-color: rgba(220,220,220,.5);
            }
            ${this.isInsideMonorepo ? `
              #timer {
                background-color: black;
                color: yellow;
                padding: 0.25em;
              }
            ` : ""}
          </style>
        </head>
        <body>
          ${body}
        </body>
      </html>
    `).call(_context4).split("\n")).call(_context3, str => (0, _trim.default)(str).call(str))).call(_context2, str => str).join("\n");
  }

  makePage404(reloadSeconds, envReport, inside, notice) {
    return this.makePage(`
      ${envReport}
      <p>missing build for package <code>embark-ui</code> ${inside}</p>
      ${notice}
      ${this.isInsideMonorepo ? `
        <p>this page will automatically reload
          in <span id="timer">${reloadSeconds}</span> seconds</p>
        <script>
          let timeLeft = ${reloadSeconds};
          const span = document.querySelector("#timer");
          const timer = window.setInterval(() => {
            if (timeLeft >= 1) {
              timeLeft -= 1;
              span.innerText = \`\${timeLeft}\`;
            }
            if (!timeLeft) {
              window.clearInterval(timer);
              window.location.reload(true);
            }
          }, 1000);
        </script>
      ` : ""}
    `);
  }

  makePage503(redirectSeconds) {
    return this.makePage(`
      <p><code>dist/server</code> is inside the monorepo at
        <code>${(0, _embarkUtils.findMonorepoPackageFromRootSync)("embark-api")}</code></p>
      <p>to access <code>embark-ui</code> in development use port
        <code>3000</code></p>
      <p>if you haven't already, please run either:</p>
      <p><code>cd ${this.monorepoRootPath} && yarn start</code><br />
        or<br />
        <code>cd ${(0, _embarkUtils.findMonorepoPackageFromRootSync)("embark-ui")} &&
          yarn start</code></p>
        <p>to instead use a static build from the monorepo, restart embark with:
          <code>EMBARK_UI_STATIC=t embark run</code></p>
      <p>this page will automatically redirect to <a id="redirect" href=""></a>
        in <span id="timer">${redirectSeconds}</span> seconds</p>
      <script>
        window.embarkApiRedirect = window.location.href.replace(
          \`http://\${window.location.hostname}:55555\`,
          \`http://\${window.location.hostname}:3000\`
        );
        document.querySelector("#redirect").href = window.embarkApiRedirect;
        let displayLink = window.embarkApiRedirect.slice(7);
        if (displayLink.endsWith(\`\${window.location.hostname}:3000/\`)) {
          displayLink = displayLink.slice(0, -1);
        }
        document.querySelector("#redirect").innerText = displayLink;
      </script>
      <script>
        let timeLeft = ${redirectSeconds};
        const span = document.querySelector("#timer");
        const timer = window.setInterval(() => {
          if (timeLeft >= 1) {
            timeLeft -= 1;
            span.innerText = \`\${timeLeft}\`;
          }
          if (!timeLeft) {
            window.clearInterval(timer);
            window.location.href = window.embarkApiRedirect;
          }
        }, 1000);
      </script>
    `);
  }

  initApp() {
    const instance = (0, _expressWs.default)((0, _express.default)());
    instance.app.use((req, _res, next) => {
      if (!this.isLogging) {
        return next();
      }

      if (!req.headers.upgrade) {
        this.embark.logger.info(`API > ${req.method} ${req.originalUrl}`);
      }

      next();
    });
    instance.app.use(_helmet.default.noCache());
    instance.app.use((0, _cors.default)());
    instance.app.use(_bodyParser.default.json());
    instance.app.use(_bodyParser.default.urlencoded({
      extended: true
    }));
    instance.app.ws("/logs", (websocket, _req) => {
      this.embark.events.on("log", (level, message) => {
        websocket.send((0, _stringify.default)({
          msg: message,
          msg_clear: message.stripColors,
          logLevel: level
        }), () => {});
      });
    });

    if (this.plugins) {
      instance.app.get("/embark-api/plugins", (_req, res) => {
        var _context5;

        res.send((0, _stringify.default)((0, _map.default)(_context5 = this.plugins.plugins).call(_context5, plugin => ({
          name: plugin.name
        }))));
      });
      const callDescriptions = this.plugins.getPluginsProperty("apiCalls", "apiCalls");
      (0, _forEach.default)(callDescriptions).call(callDescriptions, callDescription => this.registerCallDescription(instance, callDescription));
    }

    this.embark.events.on("plugins:register:api", callDescription => this.registerCallDescription(instance, callDescription));

    if (!this.isInsideMonorepo || process.env.EMBARK_UI_STATIC) {
      if (this.embark.fs.existsSync(path.join(this.embarkUiBuildDir, "index.html"))) {
        instance.app.use("/", _express.default.static(this.embarkUiBuildDir));
        instance.app.get(/^\/(?!embark-api).*$/, (_req, res) => {
          res.sendFile(path.join(this.embarkUiBuildDir, "index.html"));
        });
      } else {
        let envReport = "";
        let inside = `
          in <code>${path.dirname(this.embarkUiBuildDir)}</code>
        `;
        let notice = `
          <p>this distribution of <code>embark-ui</code> appears to be broken,
            please <a href="https://github.com/embark-framework/embark/issues">
              file an issue</a></p>
        `;

        if (this.isInsideMonorepo) {
          const embarkUiPath = (0, _embarkUtils.findMonorepoPackageFromRootSync)("embark-ui");
          envReport = `
            <p><code>process.env.EMBARK_UI_STATIC ===
              ${(0, _stringify.default)(process.env.EMBARK_UI_STATIC)}</code></p>
          `;
          inside = `
            inside the monorepo at <code>${embarkUiPath}</code>
          `;
          notice = `
            <p>to build <code>embark-ui</code> please run either:</p>
            <p><code>cd ${this.monorepoRootPath} && yarn build</code><br />
              or<br />
              <code>cd ${embarkUiPath} && yarn build</code></p>
            <p>restart <code>embark run</code> after building
              <code>embark-ui</code></p>
            <p>to instead use a live development build from the monorepo: unset
              the environment variable <code>EMBARK_UI_STATIC</code>, restart
              embark, and visit
              <a href="http://localhost:3000">http://localhost:3000</a></p>
          `;
        }

        const page404 = this.makePage404(10, envReport, inside, notice);

        const missingBuildHandler = (_req, res) => {
          res.status(404).send(page404);
        };

        instance.app.get(/^\/(?!embark-api).*$/, missingBuildHandler);
      }
    } else {
      const page503 = this.makePage503(10);

      const unavailableBuildHandler = (_req, res) => {
        res.status(503).send(page503);
      };

      instance.app.get(/^\/(?!embark-api).*$/, unavailableBuildHandler);
    }

    return instance;
  }

  registerCallDescription(instance, callDescription) {
    if (callDescription.method === "ws") {
      var _context6;

      instance.app.ws(callDescription.endpoint, (0, _bind.default)(_context6 = this.applyWSFunction).call(_context6, this, callDescription.cb));
    } else {
      var _context7;

      instance.app[callDescription.method].apply(instance.app, [callDescription.endpoint, (0, _bind.default)(_context7 = this.applyHTTPFunction).call(_context7, this, callDescription.cb)]);
    }
  }

  applyHTTPFunction(cb, req, res) {
    this.embark.events.request("authenticator:authorize", req, res, err => {
      if (err) {
        return res.send(err);
      }

      cb(req, res);
    });
  }

  applyWSFunction(cb, websocket, req) {
    this.embark.events.request("authenticator:authorize", websocket, req, err => {
      if (!err) {
        cb(websocket, req);
      }
    });
  }

}

exports.default = Server;
//# sourceMappingURL=server.js.map